@@ -1,23 +1,25 @@
+use crate::graphviz::{DiGraph, RankDir, Shape, Style};
 use std::cell::{Cell, RefCell};
 use std::collections::{BTreeSet, HashMap, HashSet, VecDeque};
 use typed_arena::Arena;
 use unicode_segmentation::UnicodeSegmentation;
 
+use crate::expr::OneOrMoreExpr;
 use crate::{
     expr::{ChoiceExpr, PrimitiveExpr, RepetitionExpr, SequenceExpr},
     scanner::Token,
     Expr,
 };
 
-#[derive(Clone, Debug, PartialEq, Eq)]
+#[derive(Clone, Copy, Debug, PartialEq, Eq, Hash)]
 pub enum TransitionKind<'a> {
-    Epsilon,
     Literal(Token<'a>),
+    Digit,
 }
 
 #[derive(Clone, Debug)]
 pub struct Transition<'a> {
-    kind: TransitionKind<'a>,
+    kind: Option<TransitionKind<'a>>,
     state: &'a State<'a>,
 }
 
@@ -53,12 +55,12 @@ impl<'a> State<'a> {
                 }
                 for transition in transitions.iter() {
                     match transition.kind {
-                        TransitionKind::Epsilon => {
+                        None => {
                             // Epsilon transition.
                             // We need to just follow this without advancing the grapheme.
                             current_states.push_back(transition.state);
                         }
-                        TransitionKind::Literal(token) => {
+                        Some(TransitionKind::Literal(token)) => {
                             if token.lexeme() == grapheme {
                                 // If character matched, push into next_states, which will
                                 // be the queue for the next character.
@@ -67,6 +69,11 @@ impl<'a> State<'a> {
                             // Otherwise, do nothing, this branch does not match and can be
                             // dropped.
                         }
+                        Some(TransitionKind::Digit) => {
+                            if grapheme.bytes().all(|b| b.is_ascii_digit()) {
+                                next_states.push_back(transition.state)
+                            }
+                        }
                     }
                 }
                 visited.insert(state.id);
@@ -83,13 +90,25 @@ impl<'a> State<'a> {
     }
 
     pub fn graphviz(&self, graph_name: &str) -> String {
-        let mut edges = Vec::new();
+        let mut digraph = DiGraph::new(graph_name);
+        digraph.rankdir(RankDir::LeftRight);
 
         let mut queue = VecDeque::new();
         let mut visited = HashSet::new();
 
         queue.push_back(self);
 
+        digraph
+            .vertex(
+                "START",
+                Style::new()
+                    .label("")
+                    .shape(Shape::None)
+                    .height(0.0)
+                    .width(0.0),
+            )
+            .edge("START", self.id, None);
+
         while let Some(state) = queue.pop_front() {
             if visited.contains(&state.id) {
                 continue;
@@ -97,38 +116,32 @@ impl<'a> State<'a> {
             let id = state.id;
             let transitions = state.transitions.borrow();
             let shape = if transitions.is_empty() {
-                "doublecircle"
+                Shape::DoubleCircle
             } else {
-                "circle"
+                Shape::Circle
             };
-            edges.push(format!("{id} [shape={shape}]"));
+            digraph.vertex(id, Style::new().shape(shape));
             for t in transitions.iter() {
                 let label = match t.kind {
-                    TransitionKind::Literal(token) => token.lexeme(),
-                    TransitionKind::Epsilon => "\u{03B5}",
+                    None => "\u{03B5}",
+                    Some(TransitionKind::Literal(token)) => token.lexeme(),
+                    Some(TransitionKind::Digit) => "\\\\d",
                 };
-                edges.push(format!("{id} -> {} [label=\"{label}\"]", t.state.id));
+                digraph.edge(id, t.state.id, Style::new().label(label));
                 queue.push_back(t.state);
             }
             visited.insert(state.id);
         }
 
-        let edges = edges.join("\n");
-
-        format!(
-            "digraph {graph_name} {{\n\
-                rankdir=LR;\n\
-            {edges}\n\
-            }}"
-        )
+        digraph.to_string()
     }
 }
 
 impl<'a> State<'a> {
-    fn transit(&self, kind: TransitionKind<'a>, state: &'a State<'a>) {
+    fn transit(&self, kind: Option<TransitionKind<'a>>, state: &'a State<'a>) {
         self.transitions
             .borrow_mut()
-            .push(Transition { kind, state })
+            .push(Transition { kind, state });
     }
 }
 
@@ -149,7 +162,7 @@ where
             continue;
         }
         for transition in transitions.iter() {
-            if let TransitionKind::Epsilon = transition.kind {
+            if transition.kind.is_none() {
                 // Follow any remaining epsilon transitions.
                 queue.push_back(transition.state);
             }
@@ -195,30 +208,54 @@ impl<'a> Compiler<'a> {
 
     fn compile_nfa(&'a self, expr: &Expr<'a>) -> NfaFragment<'a> {
         match &expr {
-            Expr::Blank(_) => panic!(),
+            Expr::Blank(_) => self.compile_blank(),
             Expr::Choice(e) => self.compile_choice(e),
             Expr::Sequence(e) => self.compile_sequence(e),
             Expr::Repetition(e) => self.compile_repetition(e),
+            Expr::OneOrMore(e) => self.compile_one_or_more(e),
             Expr::Primitive(e) => self.compile_literal(e),
+            Expr::Digit(_) => self.compile_digit(),
         }
     }
 
+    fn compile_blank(&'a self) -> NfaFragment<'a> {
+        let start = self.new_state();
+        let end = self.new_state();
+        let frag = NfaFragment::new(start, end);
+        frag.start.transit(None, end);
+
+        frag
+    }
+
     fn compile_literal(&'a self, expr: &PrimitiveExpr<'a>) -> NfaFragment<'a> {
         let start = self.new_state();
         let end = self.new_state();
         let frag = NfaFragment::new(start, end);
-        frag.start.transit(TransitionKind::Literal(expr.token), end);
+        frag.start
+            .transit(Some(TransitionKind::Literal(expr.token)), end);
 
         frag
     }
 
+    fn compile_digit(&'a self) -> NfaFragment<'a> {
+        let start = self.new_state();
+        let end = self.new_state();
+        let frag = NfaFragment::new(start, end);
+        frag.start.transit(Some(TransitionKind::Digit), end);
+
+        frag
+    }
+
+    // left.start -a-> left.end -eps-> new -eps-> right.start -b-> right.end
+    // left.start -a-> right.start -b-> right.end
+    // left.start -a-> right.start -b-> right.end -eps->
     fn compile_sequence(&'a self, expr: &SequenceExpr<'a>) -> NfaFragment<'a> {
         let left = self.compile_nfa(&*expr.start);
         let right = self.compile_nfa(&*expr.end);
-        let frag = NfaFragment::new(left.start, right.end);
-        left.end.transit(TransitionKind::Epsilon, right.start);
 
-        frag
+        left.end.transit(None, right.start);
+
+        NfaFragment::new(left.start, right.end)
     }
 
     fn compile_repetition(&'a self, regex: &RepetitionExpr<'a>) -> NfaFragment<'a> {
@@ -226,10 +263,31 @@ impl<'a> Compiler<'a> {
         let left = self.compile_nfa(&*regex.term);
         let exit = self.new_state();
         let frag = NfaFragment::new(entry, exit);
-        frag.start.transit(TransitionKind::Epsilon, left.start);
-        frag.start.transit(TransitionKind::Epsilon, frag.end);
-        left.end.transit(TransitionKind::Epsilon, left.start);
-        left.end.transit(TransitionKind::Epsilon, frag.end);
+        frag.start.transit(None, left.start);
+        frag.start.transit(None, frag.end);
+        left.end.transit(None, left.start);
+        left.end.transit(None, frag.end);
+
+        frag
+    }
+
+    fn compile_one_or_more(&'a self, regex: &OneOrMoreExpr<'a>) -> NfaFragment<'a> {
+        let initial = self.compile_nfa(&*regex.term);
+        let repetition = {
+            let entry = self.new_state();
+            let left = self.compile_nfa(&*regex.term);
+            let exit = self.new_state();
+            let frag = NfaFragment::new(entry, exit);
+            frag.start.transit(None, left.start);
+            frag.start.transit(None, frag.end);
+            left.end.transit(None, left.start);
+            left.end.transit(None, frag.end);
+
+            frag
+        };
+
+        let frag = NfaFragment::new(initial.start, repetition.end);
+        initial.end.transit(None, repetition.start);
 
         frag
     }
@@ -241,10 +299,10 @@ impl<'a> Compiler<'a> {
         let exit = self.new_state();
 
         let frag = NfaFragment::new(entry, exit);
-        frag.start.transit(TransitionKind::Epsilon, left.start);
-        left.end.transit(TransitionKind::Epsilon, frag.end);
-        frag.start.transit(TransitionKind::Epsilon, right.start);
-        right.end.transit(TransitionKind::Epsilon, frag.end);
+        frag.start.transit(None, left.start);
+        left.end.transit(None, frag.end);
+        frag.start.transit(None, right.start);
+        right.end.transit(None, frag.end);
 
         frag
     }
@@ -252,7 +310,7 @@ impl<'a> Compiler<'a> {
 
 #[derive(Debug)]
 pub struct DfaTransition<'a> {
-    lexeme: &'a str,
+    kind: TransitionKind<'a>,
     state: &'a DfaState<'a>,
 }
 
@@ -280,24 +338,22 @@ impl<'a> DfaState<'a> {
             .any(|x| x.transitions.borrow().is_empty())
     }
 
-    fn transit(&self, lexeme: &'a str, state: &'a DfaState<'a>) {
+    fn transit(&self, kind: TransitionKind<'a>, state: &'a DfaState<'a>) {
         self.transitions
             .borrow_mut()
-            .push(DfaTransition { lexeme, state })
+            .push(DfaTransition { kind, state })
     }
 
     /// move(T, a) is the set of states to which there is a transition on input symbol `a` for some NFA
     /// state in T.
-    fn find_possible_transitions(&self) -> HashMap<&'a str, Vec<&'a State<'a>>> {
-        let mut map: HashMap<&'a str, Vec<&'a State<'a>>> = HashMap::new();
+    fn find_possible_transitions(&self) -> HashMap<TransitionKind<'a>, Vec<&'a State<'a>>> {
+        let mut map: HashMap<_, Vec<&'a State<'a>>> = HashMap::new();
 
         for state in &self.nfa_states {
             let transitions = state.transitions.borrow();
             for transition in transitions.iter() {
-                if let TransitionKind::Literal(token) = transition.kind {
-                    map.entry(token.lexeme())
-                        .or_default()
-                        .push(transition.state)
+                if let Some(kind) = transition.kind {
+                    map.entry(kind).or_default().push(transition.state)
                 }
             }
         }
@@ -313,7 +369,10 @@ impl<'a> DfaState<'a> {
                 .transitions
                 .borrow()
                 .iter()
-                .find(|transition| transition.lexeme == grapheme)
+                .find(|transition| match transition.kind {
+                    TransitionKind::Literal(token) => token.lexeme() == grapheme,
+                    TransitionKind::Digit => grapheme.bytes().all(|b| b.is_ascii_digit()),
+                })
             {
                 current_state = transition.state;
             } else {
@@ -325,41 +384,65 @@ impl<'a> DfaState<'a> {
     }
 
     pub fn graphviz(&self, graph_name: &str) -> String {
-        let mut edges = Vec::new();
+        let mut digraph = DiGraph::new(graph_name);
+        digraph.rankdir(RankDir::LeftRight);
 
         let mut queue = VecDeque::new();
         let mut visited = HashSet::new();
 
         queue.push_back(self);
 
+        digraph
+            .vertex(
+                "START",
+                Style::new()
+                    .label("")
+                    .shape(Shape::None)
+                    .height(0.0)
+                    .width(0.0),
+            )
+            .edge("START", self.id, None);
+
         while let Some(state) = queue.pop_front() {
             if visited.contains(&state.id) {
                 continue;
             }
             let id = state.id;
             let shape = if state.is_accepting() {
-                "doublecircle"
+                Shape::DoubleCircle
             } else {
-                "circle"
+                Shape::Circle
             };
+
             let transitions = state.transitions.borrow();
-            edges.push(format!("{id} [shape={shape}]"));
+            let node_label = state
+                .nfa_states
+                .chunks(4)
+                .map(|x| {
+                    x.iter()
+                        .map(|x| x.id.to_string())
+                        .collect::<Vec<_>>()
+                        .join(", ")
+                })
+                .collect::<Vec<_>>()
+                .join("\n");
+
+            digraph.vertex(
+                id,
+                Style::new().label(format!("{{{node_label}}}")).shape(shape),
+            );
             for t in transitions.iter() {
-                let label = t.lexeme;
-                edges.push(format!("{id} -> {} [label=\"{label}\"]", t.state.id));
+                let label = match t.kind {
+                    TransitionKind::Literal(tok) => tok.lexeme(),
+                    TransitionKind::Digit => "\\\\d",
+                };
+                digraph.edge(id, t.state.id, Style::new().label(label));
                 queue.push_back(t.state);
             }
             visited.insert(state.id);
         }
 
-        let edges = edges.join("\n");
-
-        format!(
-            "digraph {graph_name} {{\n\
-                rankdir=LR;\n\
-            {edges}\n\
-            }}"
-        )
+        digraph.to_string()
     }
 }
 
@@ -452,6 +535,21 @@ mod test {
         match_regex!("a", "ba", false);
     }
 
+    #[test]
+    fn digit() {
+        match_regex!("\\d", "0", true);
+        match_regex!("\\d", "1", true);
+        match_regex!("\\d", "2", true);
+        match_regex!("\\d", "3", true);
+        match_regex!("\\d", "4", true);
+        match_regex!("\\d", "5", true);
+        match_regex!("\\d", "6", true);
+        match_regex!("\\d", "7", true);
+        match_regex!("\\d", "8", true);
+        match_regex!("\\d", "9", true);
+        match_regex!("\\d", "a", false);
+    }
+
     #[test]
     fn sequence() {
         match_regex!("ab", "ab", true);
@@ -481,12 +579,28 @@ mod test {
         match_regex!("a|b", "ca", false);
         match_regex!("hello|goodbye", "hello", true);
         match_regex!("hello|goodbye", "goodbye", true);
+        match_regex!("hello|", "hello", true);
+        match_regex!("hello|", "", true);
+        match_regex!("|", "", true);
+    }
+
+    #[test]
+    fn one_or_more() {
+        match_regex!("a+", "", false);
+        match_regex!("a+", "a", true);
+        match_regex!("a+", "aaa", true);
+        match_regex!("(abc)+", "abc", true);
+        match_regex!("(abc)+", "abcabc", true);
+        match_regex!("a(abc)+", "a", false);
+        match_regex!("a(abc)+", "aabc", true);
+        match_regex!("a(abc)+", "aabcabc", true);
     }
 
     #[test]
     fn mixed() {
-        match_regex!("(hello)*|goodbye", "hellohellohellohello", true);
+        match_regex!("(hello)*|g\\dodbye", "hellohellohellohello", true);
         match_regex!("(hello)*|goodbye", "hellogoodbyehellohello", false);
         match_regex!("((hello)*|goodbye)*", "hellogoodbyehellohello", true);
+        match_regex!("((hello)*|g\\dodbye)*", "hellog3odbyehellohello", true);
     }
 }
